package com.lantone.daqe.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.lantone.common.enums.IsDeleteEnum;
import com.lantone.common.util.BeanUtil;
import com.lantone.daqe.dto.GetColumnDTO;
import com.lantone.daqe.dto.GetColumnVerifyPageDTO;
import com.lantone.daqe.dto.GetRegularDTO;
import com.lantone.daqe.entity.ColumnInfo;
import com.lantone.daqe.entity.RegularInfo;
import com.lantone.daqe.facade.base.ColumnInfoFacade;
import com.lantone.daqe.facade.base.ColumnVerifyFacade;
import com.lantone.daqe.facade.base.RegularInfoFacade;
import com.lantone.daqe.vo.GetColumnVerifyPageVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @Description: 字段校验规则维护AIP
 * @author： zhanghang
 * @time: 2022/3/1 10:14
 */

@Component
public class ColumnVerifyManagementFacade {

    @Autowired
    ColumnVerifyFacade columnVerifyFacade;

    @Autowired
    ColumnInfoFacade columnInfoFacade;
    @Autowired
    RegularInfoFacade regularInfoFacade;

    /**
     * 获取字段校验规则分页列表数据
     *
     * @param getColumnVerifyPageVO
     * @return: 字段校验规则分页数据
     */
    public IPage<GetColumnVerifyPageDTO> getColumnVerifyPage(GetColumnVerifyPageVO getColumnVerifyPageVO) {
        return columnVerifyFacade.getBaseMapper().getColumnVerifyPage(getColumnVerifyPageVO);
    }

    /**
     * 下拉框获取数据库表中列数据信息
     *
     * @param
     * @return: 数据库表中列数据信息
     */
    public List<GetColumnDTO> getColumn() {
        List<ColumnInfo> columnInfoList = columnInfoFacade.getBaseMapper().selectList(new QueryWrapper<ColumnInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
        );
        List<GetColumnDTO> getColumnDTOList = BeanUtil.listCopyTo(columnInfoList, GetColumnDTO.class);
        return getColumnDTOList;
    }

    /**
     * 下拉框获取正则式名称
     *
     * @param
     * @return: 正则式名称
     */
    public List<GetRegularDTO> getRegular() {
        List<RegularInfo> regularInfoList = regularInfoFacade.getBaseMapper().selectList(new QueryWrapper<RegularInfo>()
                .eq("is_deleted", IsDeleteEnum.N.getKey())
        );
        List<GetRegularDTO> regularDTOList = BeanUtil.listCopyTo(regularInfoList, GetRegularDTO.class);
        return regularDTOList;
    }
}
