package com.lantone.daqe.facade;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lantone.common.api.CommonResult;
import com.lantone.common.exception.Asserts;
import com.lantone.common.util.BeanUtil;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.daqe.dto.ExportImportDrugErrDTO;
import com.lantone.daqe.dto.GetDrugPageDTO;
import com.lantone.daqe.dto.GetMatchingDrugPageDTO;
import com.lantone.daqe.entity.DrugInfo;
import com.lantone.daqe.entity.OperationInfo;
import com.lantone.daqe.facade.base.DrugInfoFacade;
import com.lantone.daqe.service.impl.DrugInfoServiceImpl;
import com.lantone.daqe.util.ExcelUtils;
import com.lantone.daqe.vo.AddDrugVO;
import com.lantone.daqe.vo.DelDrugByIdVO;
import com.lantone.daqe.vo.GetDrugPageVO;
import com.lantone.daqe.vo.GetMatchingDrugPageVO;
import com.lantone.daqe.vo.ImportDrugVO;
import com.lantone.daqe.vo.MatchingDrugVO;
import com.lantone.daqe.vo.UpDrugByIdVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * @Description: 药品管理-业务处理类
 * @author: rengb
 * @time: 2021/7/20 12:39
 */
@Component
public class DrugManagementFacade {

    @Autowired
    private DrugInfoFacade drugInfoFacade;

    @Autowired
    private DrugInfoServiceImpl drugInfoServiceImpl;

    public IPage<GetDrugPageDTO> getDrugPage(GetDrugPageVO getDrugPageVO) {
        Page<GetDrugPageDTO> getDrugPageDTOPage = new Page<>();

        QueryWrapper<DrugInfo> drugInfoQueryWrapper = new QueryWrapper<>();
        drugInfoQueryWrapper.eq(getDrugPageVO.getHospitalId() != null, "hospital_id", getDrugPageVO.getHospitalId());
        drugInfoQueryWrapper.like(StringUtil.isNotBlank(getDrugPageVO.getCode()), "code", getDrugPageVO.getCode());
        drugInfoQueryWrapper.like(StringUtil.isNotBlank(getDrugPageVO.getName()), "name", getDrugPageVO.getName());
        drugInfoQueryWrapper.like(StringUtil.isNotBlank(getDrugPageVO.getApprovalNum()), "approval_num", getDrugPageVO.getApprovalNum());
        drugInfoQueryWrapper.like(StringUtil.isNotBlank(getDrugPageVO.getStandard()), "standard", getDrugPageVO.getStandard());
        drugInfoQueryWrapper.like(StringUtil.isNotBlank(getDrugPageVO.getDosageForm()), "standard", getDrugPageVO.getDosageForm());
        if (StringUtil.isNotBlank(getDrugPageVO.getIsMapping())) {
            if (getDrugPageVO.getIsMapping().equals("0")) {
                drugInfoQueryWrapper.isNull("standard");
            } else if (getDrugPageVO.getIsMapping().equals("1")) {
                drugInfoQueryWrapper.isNotNull("standard");
            }
        }
        Page<DrugInfo> drugInfoPage = new Page<>(getDrugPageVO.getCurrent(), getDrugPageVO.getSize());
        drugInfoFacade.page(drugInfoPage, drugInfoQueryWrapper);
        BeanUtil.copyProperties(drugInfoPage, getDrugPageDTOPage);
        List<GetDrugPageDTO> getDrugPageDTOList = BeanUtil.listCopyTo(drugInfoPage.getRecords(), GetDrugPageDTO.class);
        getDrugPageDTOList.forEach(i -> {
            i.setIsMapping(StringUtil.isBlank(i.getIsMapping()) ? "未匹配" : "已匹配");
        });

        getDrugPageDTOPage.setRecords(getDrugPageDTOList);
        return getDrugPageDTOPage;
    }

    /**
     * 新增药品 业务处理
     *
     * @param addDrugVO
     * @return 是否新增成功
     */
    public boolean addDrug(AddDrugVO addDrugVO) {
        DrugInfo drugInfo = new DrugInfo();
        BeanUtil.copyProperties(addDrugVO, drugInfo);
        if (drugInfoFacade.isExist(drugInfo)) {
            Asserts.fail("该诊断已存在！");
        }
        return drugInfoFacade.save(drugInfo);
    }

    /**
     * 通过id修改药品
     *
     * @param upDrugByIdVO
     * @return 是否修改成功
     */
    public Boolean upDrugById(UpDrugByIdVO upDrugByIdVO) {
        if (drugInfoFacade.getById(upDrugByIdVO.getId()) == null) {
            Asserts.fail("该药品不存在！");
        }
        DrugInfo drugInfo = new DrugInfo();
        BeanUtil.copyProperties(upDrugByIdVO, drugInfo);
        return drugInfoFacade.updateById(drugInfo);
    }

    /**
     * 通过id删除药品
     *
     * @param delDrugByIdVO
     * @return 是否删除成功
     */
    public Boolean delDrugById(DelDrugByIdVO delDrugByIdVO) {
        if (drugInfoFacade.getById(delDrugByIdVO.getId()) == null) {
            Asserts.fail("该药品不存在！");
        }
        return drugInfoFacade.removeById(delDrugByIdVO.getId());
    }

    /**
     * 匹配药品信息
     *
     * @param matchingDrugVO
     * @Return java.lang.Boolean
     */
    public Boolean matchingDrug(MatchingDrugVO matchingDrugVO) {

        DrugInfo drugInfo = new DrugInfo();
        BeanUtils.copyProperties(matchingDrugVO, drugInfo);
        drugInfo.setStandard(matchingDrugVO.getStandard() + "_" + matchingDrugVO.getForm());
        return drugInfoFacade.updateById(drugInfo);
    }

    /**
     *药品匹配信息导入
     *
     * @param response
     * @param file
     * @Return void
     */
    public void importDrug(HttpServletResponse response, MultipartFile file) {

        List<ImportDrugVO> operationExcelVOS = ExcelUtils.importExcel(file, 0, 1, ImportDrugVO.class);
        if (ListUtil.isEmpty(operationExcelVOS)) {
            Asserts.fail("Excel文件为空");
        }
        //数据规范校验
        List<ExportImportDrugErrDTO> errExports = new ArrayList<>();
        if (checkData(operationExcelVOS, errExports)) {
            String fileName = "药品匹配信息导入异常.xls";
            ExcelUtils.exportExcelUser(errExports, null, "sheet1", ExportImportDrugErrDTO.class, fileName, response);
        } else {
            //去重
            List<ImportDrugVO> temp = delRepeat(operationExcelVOS);
            List<DrugInfo> insert = new ArrayList<>();
            //导入数据转换
            temp.stream().forEach(importDrugVO -> {
                DrugInfo drugInfo = new DrugInfo();
                BeanUtils.copyProperties(importDrugVO,drugInfo);
                drugInfo.setStandard(importDrugVO.getStandard()+"_"+importDrugVO.getForm());
            });
            boolean out = drugInfoServiceImpl.saveBatch(insert);
            CommonResult<String> outMsg = null;
            response.setContentType("text/html;charset=utf-8");
            if (out) {
                outMsg = CommonResult.success(String.format("模板成功导入%s条数据", insert.size()));
                try {
                    response.getWriter().println(JSONObject.toJSONString(outMsg));
                    response.flushBuffer();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } else {
                Asserts.fail("数据插入失败");
            }
        }
    }


    /**
     * 数据校验
     *
     * @param importDrugVOS 导入数据
     * @param errExports 导出异常i
     * @Return java.lang.Boolean
     */
    private Boolean checkData(List<ImportDrugVO> importDrugVOS, List<ExportImportDrugErrDTO> errExports) {
        AtomicBoolean errorFlog = new AtomicBoolean(false);
        importDrugVOS.stream().forEach(importDrugVO -> {
            ExportImportDrugErrDTO exportImportDrugErrDTO = new ExportImportDrugErrDTO();
            BeanUtil.copyProperties(importDrugVO, exportImportDrugErrDTO);
            StringBuffer errMsg = new StringBuffer();
            //数据校验
            if (StringUtil.isEmpty(importDrugVO.getName())) {
                errMsg.append("医院手术/操作名称为空").append(";");
                errorFlog.set(true);
            }
            if (StringUtil.isEmpty(importDrugVO.getStandard())) {
                errMsg.append("标准手术/操作名称为空").append(";");
                errorFlog.set(true);
            }
            exportImportDrugErrDTO.setErrMsg(errMsg.toString());
            errExports.add(exportImportDrugErrDTO);
        });
        return errorFlog.get();
    }

    /**
     * 去重
     *
     * @param importDrugVOS
     * @Return java.util.List<com.lantone.daqe.vo.ImportDrugVO>
     */
    private List<ImportDrugVO> delRepeat(List<ImportDrugVO> importDrugVOS) {

        //通过标准词去重
        LinkedHashMap<String, ImportDrugVO> tempMap = importDrugVOS.stream()
                .collect(Collectors.toMap(ImportDrugVO::getName, ImportDrugVO -> ImportDrugVO, (k1, k2) -> k1, LinkedHashMap::new));
        return tempMap.values().stream().collect(Collectors.toList());
    }

    /**
     *获取药品匹配分页列表
     *
     * @param getMatchingDrugPageVO
     * @Return com.baomidou.mybatisplus.core.metadata.IPage<com.lantone.daqe.dto.GetMatchingDrugPageDTO>
     */
    public IPage<GetMatchingDrugPageDTO> getMatchingDrugPage(GetMatchingDrugPageVO getMatchingDrugPageVO) {

        List<GetMatchingDrugPageDTO> getMatchingDrugPageDTOList = new ArrayList<>();
        //数据整理组装
        IPage<GetMatchingDrugPageDTO> drugPages = drugInfoServiceImpl.getBaseMapper().getMatchingDrugPage(getMatchingDrugPageVO);
        for(GetMatchingDrugPageDTO drug:drugPages.getRecords()){
            //目前先按照_切割
            drug.setForm(drug.getStandard().split("_")[1]);
            getMatchingDrugPageDTOList.add(drug);
        }
        return drugPages.setRecords(getMatchingDrugPageDTOList);
    }
}