package com.lantone.daqe.facade;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.lantone.common.api.CommonResult;
import com.lantone.common.exception.Asserts;
import com.lantone.common.util.BeanUtil;
import com.lantone.common.util.ListUtil;
import com.lantone.common.util.StringUtil;
import com.lantone.daqe.dto.ExportImportOperationDTO;
import com.lantone.daqe.dto.GetMatchingOperationPageDTO;
import com.lantone.daqe.dto.GetOperationPageDTO;
import com.lantone.daqe.entity.OperationInfo;
import com.lantone.daqe.facade.base.OperationInfoFacade;
import com.lantone.daqe.service.impl.OperationInfoServiceImpl;
import com.lantone.daqe.util.ExcelUtils;
import com.lantone.daqe.vo.AddOperationVO;
import com.lantone.daqe.vo.DelOperationByIdVO;
import com.lantone.daqe.vo.GetMatchingOperationVO;
import com.lantone.daqe.vo.GetOperationPageVO;
import com.lantone.daqe.vo.ImportOperationVO;
import com.lantone.daqe.vo.MatchingOperationVO;
import com.lantone.daqe.vo.UpOperationByIdVO;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.stream.Collectors;

/**
 * @Description: 手术管理-业务处理类
 * @author: rengb
 * @time: 2021/7/20 12:39
 */
@Component
public class OperationManagementFacade {

    @Autowired
    private OperationInfoFacade operationInfoFacade;

    @Autowired
    private OperationInfoServiceImpl operationInfoServiceImpl;

    public IPage<GetOperationPageDTO> getOperationPage(GetOperationPageVO getOperationPageVO) {
        Page<GetOperationPageDTO> getOperationPageDTOPage = new Page<>();

        QueryWrapper<OperationInfo> operationInfoQueryWrapper = new QueryWrapper<>();
        operationInfoQueryWrapper.eq(getOperationPageVO.getHospitalId() != null, "hospital_id", getOperationPageVO.getHospitalId());
        operationInfoQueryWrapper.like(StringUtil.isNotBlank(getOperationPageVO.getCode()), "code", getOperationPageVO.getCode());
        operationInfoQueryWrapper.like(StringUtil.isNotBlank(getOperationPageVO.getName()), "name", getOperationPageVO.getName());
        operationInfoQueryWrapper.like(StringUtil.isNotBlank(getOperationPageVO.getStandard()), "standard", getOperationPageVO.getStandard());
        if (StringUtil.isNotBlank(getOperationPageVO.getIsMapping())) {
            if (getOperationPageVO.getIsMapping().equals("0")) {
                operationInfoQueryWrapper.isNull("standard");
            } else if (getOperationPageVO.getIsMapping().equals("1")) {
                operationInfoQueryWrapper.isNotNull("standard");
            }
        }
        Page<OperationInfo> operationInfoPage = new Page<>(getOperationPageVO.getCurrent(), getOperationPageVO.getSize());
        operationInfoFacade.page(operationInfoPage, operationInfoQueryWrapper);
        BeanUtil.copyProperties(operationInfoPage, getOperationPageDTOPage);
        List<GetOperationPageDTO> getOperationPageDTOList = BeanUtil.listCopyTo(operationInfoPage.getRecords(), GetOperationPageDTO.class);
        getOperationPageDTOList.forEach(i -> {
            i.setIsMapping(StringUtil.isBlank(i.getIsMapping()) ? "未匹配" : "已匹配");
        });

        getOperationPageDTOPage.setRecords(getOperationPageDTOList);
        return getOperationPageDTOPage;
    }

    /**
     * 新增手术 业务处理
     *
     * @param addOperationVO
     * @return 是否新增成功
     */
    public boolean addOperation(AddOperationVO addOperationVO) {
        OperationInfo operationInfo = new OperationInfo();
        BeanUtil.copyProperties(addOperationVO, operationInfo);
        if (operationInfoFacade.isExist(operationInfo)) {
            Asserts.fail("该诊断已存在！");
        }
        return operationInfoFacade.save(operationInfo);
    }

    /**
     * 通过id修改手术
     *
     * @param upOperationByIdVO
     * @return 是否修改成功
     */
    public Boolean upOperationById(UpOperationByIdVO upOperationByIdVO) {
        if (operationInfoFacade.getById(upOperationByIdVO.getId()) == null) {
            Asserts.fail("该药品不存在！");
        }
        OperationInfo operationInfo = new OperationInfo();
        BeanUtil.copyProperties(upOperationByIdVO, operationInfo);
        return operationInfoFacade.updateById(operationInfo);
    }

    /**
     * 通过id删除手术
     *
     * @param delOperationByIdVO
     * @return 是否删除成功
     */
    public Boolean delOperationById(DelOperationByIdVO delOperationByIdVO) {
        if (operationInfoFacade.getById(delOperationByIdVO.getId()) == null) {
            Asserts.fail("该药品不存在！");
        }
        return operationInfoFacade.removeById(delOperationByIdVO.getId());
    }

    /**
     * 匹配手术信息
     *
     * @param matchingOperationVO
     * @Return java.lang.Boolean
     */
    public Boolean matchingOperation(MatchingOperationVO matchingOperationVO) {
        OperationInfo operationInfo = new OperationInfo();
        BeanUtils.copyProperties(matchingOperationVO, operationInfo);
        return operationInfoFacade.updateById(operationInfo);
    }

    /**
     * 医院手术标准词匹配信息导入
     *
     * @param response
     * @param file
     * @Return void
     */
    public void importOperation(HttpServletResponse response, MultipartFile file) {

        List<ImportOperationVO> operationExcelVOS = ExcelUtils.importExcel(file, 0, 1, ImportOperationVO.class);
        if (ListUtil.isEmpty(operationExcelVOS)) {
            Asserts.fail("Excel文件为空");
        }
        //数据规范校验
        List<ExportImportOperationDTO> errExports = new ArrayList<>();
        if (checkData(operationExcelVOS, errExports)) {
            String fileName = "医院手术标准词匹配信息导入异常.xls";
            ExcelUtils.exportExcelUser(errExports, null, "sheet1", ExportImportOperationDTO.class, fileName, response);
        } else {
            //去重
            List<ImportOperationVO> temp = delRepeat(operationExcelVOS);
            //导入
            List<OperationInfo> insert = BeanUtil.listCopyTo(temp, OperationInfo.class);
            boolean out = operationInfoServiceImpl.saveBatch(insert);
            CommonResult<String> outMsg = null;
            response.setContentType("text/html;charset=utf-8");
            if (out) {
                outMsg = CommonResult.success(String.format("模板成功导入%s条数据", insert.size()));
                try {
                    response.getWriter().println(JSONObject.toJSONString(outMsg));
                    response.flushBuffer();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            } else {
                Asserts.fail("数据插入失败");
            }
        }
    }


    /**
     * 数据校验
     *
     * @param importOperationVOS 导入数据
     * @param errExports         导出异常
     * @Return java.lang.Boolean
     */
    private Boolean checkData(List<ImportOperationVO> importOperationVOS, List<ExportImportOperationDTO> errExports) {
        AtomicBoolean errorFlog = new AtomicBoolean(false);
        importOperationVOS.stream().forEach(importOperationVO -> {
            ExportImportOperationDTO exportImportOperationDTO = new ExportImportOperationDTO();
            BeanUtil.copyProperties(importOperationVO, exportImportOperationDTO);
            StringBuffer errMsg = new StringBuffer();
            //数据校验
            if (StringUtil.isEmpty(importOperationVO.getName())) {
                errMsg.append("医院手术/操作名称为空").append(";");
                errorFlog.set(true);
            }
            if (StringUtil.isEmpty(importOperationVO.getStandard())) {
                errMsg.append("标准手术/操作名称为空").append(";");
                errorFlog.set(true);
            }
            exportImportOperationDTO.setErrMsg(errMsg.toString());
            errExports.add(exportImportOperationDTO);
        });
        return errorFlog.get();
    }

    /**
     * 去重
     *
     * @param importOperationVOS
     * @Return java.util.List<com.lantone.daqe.vo.ImportOperationVO>
     */
    private List<ImportOperationVO> delRepeat(List<ImportOperationVO> importOperationVOS) {

        //通过标准词去重
        LinkedHashMap<String, ImportOperationVO> tempMap = importOperationVOS.stream()
                .collect(Collectors.toMap(ImportOperationVO::getName, ImportOperationVO -> ImportOperationVO, (k1, k2) -> k1, LinkedHashMap::new));
        return tempMap.values().stream().collect(Collectors.toList());
    }

    /**
     * 获取手术匹配信息分页列表
     *
     * @param getMatchingOperationVO
     * @Return com.baomidou.mybatisplus.core.metadata.IPage<com.lantone.daqe.dto.GetMatchingOperationPageDTO>
     */
    public IPage<GetMatchingOperationPageDTO> getMatchingOperationPage(GetMatchingOperationVO getMatchingOperationVO) {

        return operationInfoFacade.getBaseMapper().getMatchingOperationPage(getMatchingOperationVO);
    }
}